@@ -1,4 +1,4 @@
-package Algorithm::CheckDigits::M10_010;
+package Algorithm::CheckDigits::M11_010;
 
 use 5.006;
 use strict;
@@ -24,7 +24,7 @@ our @EXPORT_OK = ( 'new', @{ $EXPORT_TAGS{'all'} } );
 
 our @EXPORT = ();
 
-my @items = ( 0,9,4,6,8,2,7,1,3,5 );
+my @weight = ( 5, 4, 3, 2, 7, 6, 5, 4, 3, 2 );
 
 sub new {
 	my $proto = shift;
@@ -37,55 +37,49 @@ sub new {
 
 sub is_valid {
 	my ($self,$number) = @_;
-	if ($number =~ /^(\d\d-?\d{8})-?(\d)$/) {
-		return $2 == $self->_compute_checkdigit($1);
+	if ($number =~ /^([0-9.]+)(\d)$/) {
+		return $2 == $self->_compute_checkdigits($1);
 	}
 	return ''
 } # is_valid()
 
 sub complete {
 	my ($self,$number) = @_;
-	if ($number =~ /^\d\d-?\d{8}-?$/) {
-		return  $number . $self->_compute_checkdigit($number);
+	if ($number =~ /^([0-9.]+)$/) {
+		return "$1" . $self->_compute_checkdigits($1);
 	}
 	return '';
 } # complete()
 
 sub basenumber {
 	my ($self,$number) = @_;
-	if ($number =~ /^(\d\d-?\d{8}-?)(\d)$/) {
-		return $1 if ($2 == $self->_compute_checkdigit($1));
+	if ($number =~ /^([0-9.]+)(\d)$/) {
+		return $1 if ($2 == $self->_compute_checkdigits($1));
 	}
 	return '';
 } # basenumber()
 
 sub checkdigit {
 	my ($self,$number) = @_;
-	if ($number =~ /^(\d\d-?\d{8})-?(\d)$/) {
-		return $2 if ($2 == $self->_compute_checkdigit($1));
+	if ($number =~ /^([0-9.]+)(\d)$/) {
+		return $2 if ($2 == $self->_compute_checkdigits($1));
 	}
 	return '';
 } # checkdigit()
 
-sub _compute_checkdigit {
+sub _compute_checkdigits {
 	my $self   = shift;
 	my $number = shift;
 
-	if ($number =~ /^\d\d-?\d{8}-?$/) {
+	$number =~ s/\.//g;
 
-		$number =~ s/-//g;
-		my @digits = split(//,$number);
-		my $sum    = 0;
-		my $cf     = 0;
-
-		for (my $i = 0; $i <= $#digits; $i++) {
-
-			$cf = $items[($digits[$i] + $cf) % 10];
-
-		}
-		return (10 - $cf) % 10;
+	my @digits = split(//,$number);
+	my $sum = 0;
+	for (my $i = 0; $i <= $#digits; $i++) {
+		$sum += $weight[$i] * $digits[$i];
 	}
-	return -1;
+	$sum %= 11;
+	return $sum == 0 ? 0 : ($sum == 1 ? '' : 11 - $sum);
 } # _compute_checkdigit()
 
 # Preloaded methods go here.
@@ -95,26 +89,26 @@ __END__
 
 =head1 NAME
 
-CheckDigits::M10_010 - compute check digits for Postscheckkonti (CH)
+CheckDigits::M11_010 - compute check digits AHV number (CH)
 
 =head1 SYNOPSIS
 
   use CheckDigits;
 
-  $pck = CheckDigits('postcheckkonti');
+  $ahv = CheckDigits('ahv_ch');
 
-  if ($pck->is_valid('85-12345678-7')) {
+  if ($ahv->is_valid('123.45.678.113')) {
 	# do something
   }
 
-  $cn = $pck->complete('85-12345678');
-  # $cn = '85-12345678-7'
+  $cn = $ahv->complete('123.45.678.11');
+  # $cn = '123.45.678.113'
 
-  $cd = $pck->checkdigit('85-12345678-7');
-  # $cd = '7'
+  $cd = $ahv->checkdigit('123.45.678.113');
+  # $cd = '3'
 
-  $bn = $pck->basenumber('85-12345678-7');
-  # $bn = '85-12345678'
+  $bn = $ahv->basenumber('123.45.678.113');
+  # $bn = '123.45.678.11';
   
 =head1 DESCRIPTION
 
@@ -124,19 +118,21 @@ CheckDigits::M10_010 - compute check digits for Postscheckkonti (CH)
 
 =item 1
 
-The sequence of digits is processed left to right. For the first digit
-we assume a carry forward of 0.
+Beginning left every digit is weighted with 5,4,3,2,7,6,5,4,3,2.
 
 =item 2
 
-For each digit d(i) the carry forward cf(i) is the digit at the
-the position p in the sequence ( 0, 9, 4, 6, 8, 2, 7, 1, 3, 5 ), where
-p is (d(i) + cf(i-1)) modulo 10.
+The weighted digits are added.
 
 =item 3
 
-The check digit is the difference of the sum from step 3 to the next
-multiple of 10.
+The sum from step 2 is taken modulo 11.
+
+=item 4
+
+The checkdigit is 11 minus the sum from step 3. Is the difference 10,
+the number won't be taken. If the difference is 11, the checkdigit is
+0.
 
 =back
 
@@ -146,18 +142,19 @@ multiple of 10.
 
 =item is_valid($number)
 
-Returns true only if C<$number> consists solely of numbers and the last digit
-is a valid check digit according to the algorithm given above.
+Returns true only if C<$number> consists solely of numbers and hyphens
+and the two digits in the middle
+are valid check digits according to the algorithm given above.
 
 Returns false otherwise,
 
 =item complete($number)
 
-The check digit for C<$number> is computed and concatenated to the end
-of C<$number>.
+The check digit for C<$number> is computed and inserted into the
+middle of C<$number>.
 
 Returns the complete number with check digit or '' if C<$number>
-does not consist solely of digits and spaces.
+does not consist solely of digits, hyphens and spaces.
 
 =item basenumber($number)
 
@@ -168,8 +165,8 @@ Return '' otherwise.
 
 =item checkdigit($number)
 
-Returns the checkdigit of C<$number> if C<$number> has a valid check
-digit.
+Returns the check digits of C<$number> if C<$number> has valid check
+digits.
 
 Return '' otherwise.
 
@@ -187,6 +184,6 @@ Mathias Weidner, E<lt>mathias@weidner.in-bad-schmiedeberg.deE<gt>
 
 L<perl>,
 L<CheckDigits>,
-F<www.pruefziffernberechnung.de>.
+F<www.pruefziffernberechnung.de>,
 
 =cut
